@@ -18,6 +18,123 @@
 <div class="main1">
 <dl>
 
+<dt>1.4.2 (2025/05/17)
+<dd><ul>
+<li>m88k: FP precise 例外時にソースレジスタが 0 の場合の FPHS[12]
+ レジスタを修正。
+ <q>Fix the FPHS[12] registers if any source register contains 0
+ when the FP precise exception occurs.</q>
+<li>m88k: テーブルサーチは暗黙で CI 扱いになるところを実装。
+ <q>Implement a table search operation with CI.</q>
+<li>m88k: CI アクセスでキャッシュヒットするとエントリが無効化されるところを実装。
+この2点で、ver 0.7.1 以降 OpenBSD がまれに panic していたのが直ると思います。
+ <q>Implement that a cache hit invalidates the cache entry if the accesss is CI.
+ These two changes should fix an occasional OpenBSD panic since ver 0.7.1.</q>
+<li>m88k: CI 領域に対する xmem 命令の動作を修正。
+ <q>Fix the behavior of the xmem instruction in a cache-inhibited region.</q>
+<li>m88k: バススヌープでキャッシュの状態が変わらない場合があったのを修正。
+ <q>Fix an issue that bus snooping didn't change cache status in some
+ case.</q>
+<li>m88k: m88200 の Cache Copyback {,and Invalidate} コマンド実行後の
+キャッシュステータスを修正。
+ <q>Fix the cache status on 88200 Cache Copyback {,and Invalidate} commands.</q>
+</ul><p>
+
+<dt>1.4.1 (2025/04/30)
+<dd><ul>
+<li>m88k: 例外優先度をいろいろ修正。
+ <q>Fix several exception priorities.</q>
+<li>m88k: 不当命令例外を実装。
+ <q>Implement the illegal instruction exception.</q>
+<li>m88k: データアクセス例外発生時の DMx レジスタをいくつか修正。
+ <q>Fix several DMx registers upon a data access exception.</q>
+<li>m88k: ld.d 命令の第2ワードで例外が発生した時の動作を修正。
+ <q>Fix the behavior of the ld.d instruction when an exception occurred on
+ the second word.</q>
+<li>m88k: PID, SXIP レジスタが書き込み可能になっていたのを修正。
+ <q>Fix that PID and SXIP registers were writeable.</q>
+<li>m88k: m88200 の内部の BATC/PATC サーチ方法を ver 0.6.1 のものに戻す。
+ <q>Revert the search method inside the BATC/PATC to that of ver 0.6.1.</q>
+<li>vm: 内蔵 LUNA-88K PROM の割り込みハンドラを整備。
+ OpenBSD/luna88k の /boot の hexdump コマンドがバスエラー領域でハングアップ
+ していたのが直ります。
+ <q>Implement an interrupt handler in the builtin emulated PROM on LUNA-88K.
+ This fixes the issue where the hexdump command on the OpenBSD/luna88k boot
+ loader was hanging when accessing the bus error region.</q>
+<li>vm: LUNA-88K のメモリマップを全体的に更新。実機に近づける。
+ <q>Modify and adjust the memory map on LUNA-88K.</q>
+<li>vm: SCSI バスでセレクションフェーズからメッセージアウトフェーズへの移行を
+ 実装。
+ これで UniOS Mach 起動中に nono が不正終了していたのは直ります(まだ動きません)。
+ <q>Implement the SCSI bus transition from the selection phase to
+ the message out phase.
+ This fixes the abnormal termination while booting UniOS Mach.
+ (But it doesn't work yet.)</q>
+<li>vm: 書き込み禁止の SCSI イメージファイルの扱いを変更。
+ <q>Change the handling of write-protected SCSI image files.</q>
+<li>vm: ソフトウェアキーボードを LUNA-88K に対応(?)。
+それに伴い LUNA-I、X68030 の一部の内部キーコードも変更。
+ <q>Support the LUNA-88K software keyboard(?).  As a result of this,
+ several internal keycodes on LUNA-I and X68030 are also moved.</q>
+<li>debugger: ブレークポイントモニタを m88k に対応。
+ <q>The breakpoint monitor now supports m88k.</q>
+<li>debugger: ブレークポイントモニタの命令ブレークポイントを 1バイト命令
+(HD64180) にも対応。
+ <q>The breakpoint monitor now supports 1-byte instructions like HD64180.</q>
+<li>debugger: ブレークポイントの CPU 名が正しくない場合に不正終了する場合があったのを修正。
+ <q>Fix the abnormal termination when a breakpoint has an invalid CPU name.</q>
+<li>debugger: ベクタテーブルを LUNA-88K に対応。
+ <q>The vector table now supports LUNA-88K.</q>
+<li>debugger: bv コマンドの書式を変更。
+ <q>Change the syntax of the bv command.</q>
+<li>debugger: bi コマンドで一部のニーモニック表記を実装。
+ <q>The bi command supports some mnemonics.</q>
+<li>debugger: --bi オプション、--bv オプションを実装。
+ <q>Implement --bi and --bv options.</q>
+<li>util/nvramedit: NVRAM.DAT のファイルサイズを修正。
+ <q>Fix the file size of NVRAM.DAT.</q>
+</ul><p>
+
+<dt>1.4.0 (2025/03/30)
+<dd><ul>
+<li>vm(New): HD647180 ASCI (非同期シリアル) を実装。
+ <q>Implement HD647180 ASCI (Asynchronous Serial Communication Interface).</q>
+<li>vm(Fix): X68030 キーボード LED の変更がソフトウェアキーボードに
+反映されないことがあったのを修正。
+ <q>Fix an issue that keyboard LED changes may not be reflected in the
+ software keyboard window on X68030.</q>
+<li>host(New): ネットワークとシリアルのホストドライバの実行時切り替えを実装。
+ <q>Implement run-time switching of the network and serial host driver.</q>
+<li>app(Update): <tt>hostcom-driver=cons</tt> を
+<tt>hostcom-driver=console</tt> に変更。(<tt>cons</tt> も引き続き使用可能)
+ <q>Change <tt>hostcom-driver=cons</tt> to <tt>hostcom-driver=console</tt>
+ (<tt>cons</tt> is still available).</q>
+<li>GUI(Fix): マウスモードを抜けてもタイトルが戻らない場合があったのを修正。
+ <q>Fix an issue that the title might not change after leaving mouse mode.</q>
+<li>GUI(Fix): ステータスパネルの幅によっては不正終了していたのを修正。
+ <q>Fix abnormal terminations caused by status panel width.</q>
+<li>GUI(Fix): ログモニタ表示中にフォントサイズ変更すると不正終了する場合が
+あったのを修正。
+ <q>Fix an issue that changing the font size while the log monitor is
+ displayed might cause an abnormal termination.</q>
+<li>GUI(Fix): ログモニタ、LCD モニタがフォントサイズ変更でレイアウトが
+崩れる場合があったのを修正。
+ <q>Fix an issue that changing the font size might break layout
+ on the log monitor and LCD monitor.</q>
+<li>GUI(Fix): ウィンドウの描画で不正終了する可能性があったのを修正。
+ <q>Fix several issues that drawing window might cause abnormal termination.</q>
+<li>GUI(Fix): メモリモニタのボタンサイズを調整。
+ <q>Adjust button size on the memory monitor.</q>
+<li>GUI(Update): メイン画面縮小時のレイアウトを調整。
+ <q>Adjust main window layout when the window is shrunk.</q>
+<li>GUI(Update): メモリモニタ、ビットマップモニタの初期サイズを調整。
+ <q>Adjust initial size of memory monitorand bitmap monitor.</q>
+<li>GUI(Update): メモリモニタ、ログモニタ、履歴モニタ、ベクタモニタの
+ウィンドウの伸縮を1行単位に変更。
+ <q>Adjust window stretch of memory monitor, log monitor, history monitor
+and vector monitor.</q>
+</ul><p>
+
 <dt>1.3.0 (2025/02/26)
 <dd><ul>
 <li>vm(New): VirtIO SCSI を実装。
