@@ -26,7 +26,7 @@ class Driver : public Object
  public:
 	~Driver() override;
 
-	virtual bool InitDriver();
+	virtual bool InitDriver(bool startup = true);
 
 	// ドライバ固有の Dispatch。 udata を返す。
 	// 必要に応じて udata を変更しても良い。
@@ -44,10 +44,19 @@ class Driver : public Object
 	// 親ホストデバイス(基本クラス)を取得。
 	HostDevice *GetHostDev() const { return hostdev; }
 
+	// InitDriver() 中の(主に最後の)エラーメッセージ。
+	// 各ドライバはエラーが発生しても各自で warnx による表示は基本的にせず、
+	// ここにエラーメッセージをセットする。必要になった時点で HostDevice が
+	// 最後の一つを warnx() で表示する。
+	// なお、おそらくそれだけでは大抵何も分からないのでこれとは別に
+	// ログレベル 1 で putmsg() は充実させておくこと。
+	std::string errmsg {};
+
  protected:
 	// 親ホストデバイス
 	HostDevice *hostdev {};
 
-	// ドライバ名 ("None" とか)
+	// ドライバ名 ("none" とか)。
+	// 設定ファイルに合わせてすべて小文字。
 	const char *drivername {};
 };
