@@ -105,9 +105,13 @@ MainApp::ShowHelp(bool all) const
 
 	if (all) {
 		printf("\n(options for developers)\n");
-		p("-b [<cpu>,]<addr>[,<skip>]");
+		p("--b [<cpu>,]<addr>[,<skip>] (or -b)");
 		p("                    set breakpoint. <cpu> is either 'main'(default) or 'xp'");
+		p("--bi [<cpu>,]<inst>[:<mask>][,<skip>]");
+		p("                    set instruction breakpoint.");
 		p("--bi-exg            set instruction breakpoint on 'exg sp,sp'");
+		p("--bv [<cpu>,]<vec1>[-<vec2>][,<skip>]");
+		p("                    set vector breakpoint.");
 		p("-C                  output log to console");
 		p("--console-log <file> specify the console output log");
 		p("-d                  debugger prompt on startup");
@@ -126,7 +130,9 @@ MainApp::ShowHelp(bool all) const
 // 0x80 から始めておく。
 enum {
 	OPTstart = 0x80 - 1,
+	OPT_bi,
 	OPT_bi_exg,
+	OPT_bv,
 	OPT_console_log,
 	OPT_create_sram,
 	OPT_fd0,
@@ -140,7 +146,10 @@ enum {
 	OPT_show_hostnet,
 };
 static struct option longopts[] = {
+	{ "b",				required_argument,	NULL,	'b' },
+	{ "bi",				required_argument,	NULL,	OPT_bi },
 	{ "bi-exg",			no_argument,		NULL,	OPT_bi_exg },
+	{ "bv",				required_argument,	NULL,	OPT_bv },
 	{ "console-log",	required_argument,	NULL,	OPT_console_log },
 	{ "create-sram",	no_argument,		NULL,	OPT_create_sram },
 	{ "fd0",			required_argument,	NULL,	OPT_fd0 },
@@ -263,7 +272,9 @@ MainApp::Init1a(int ac, char *av[])
 		gConfig->Update("ram-size=16", "-S");
 		gConfig->Update("luna-video-plane=1", "-S");
 		// ホスト側には干渉しない。
-		gConfig->Update("hostcom-driver=none", "-S");
+		gConfig->Update("hostcom0-driver=none", "-S");
+		gConfig->Update("hostcom1-driver=none", "-S");
+		gConfig->Update("hostcom2-driver=none", "-S");
 		gConfig->Update("hostnet-driver=none", "-S");
 	}
 
@@ -566,10 +577,18 @@ MainApp::ParseOpt(int ac, char *av[])
 			config_options.emplace_back(optarg, "-V");
 			break;
 
+		 case OPT_bi:
+			debug_breakinst.push_back(optarg);
+			break;
+
 		 case OPT_bi_exg:
 			debug_breakinst_exg = true;
 			break;
 
+		 case OPT_bv:
+			debug_breakvec.push_back(optarg);
+			break;
+
 		 case OPT_console_log:
 			// この時点ではコンソールを持つ VM かどうかが分からないので
 			// コンソールのない設定でもエラーを出さない(出せない)。
@@ -990,33 +1009,22 @@ MainApp::SetLogopt1(const std::string& item, std::string *errmsg)
 		}
 		return true;
 	}
-	// "fdd" なら "fdd*" 全部にする
-	if (name == "fdd") {
-		for (auto obj : gMainApp.GetObjects()) {
-			if (strncasecmp(obj->GetName().c_str(), "fdd", 3) == 0) {
-				obj->SetLogLevel(level);
-			}
-		}
-		return true;
-	}
-	// "bankram" も "bankram[01]" 両方にする
-	if (name == "bankram") {
-		for (auto obj : gMainApp.GetObjects()) {
-			if (strncasecmp(obj->GetName().c_str(), "bankram", 7) == 0) {
-				obj->SetLogLevel(level);
-			}
-		}
-		return true;
-	}
-	// "gfpic" も "gfpic*" 全部にする
-	if (name == "gfpic") {
-		for (auto obj : gMainApp.GetObjects()) {
-			if (strncasecmp(obj->GetName().c_str(), "gfpic", 5) == 0) {
-				obj->SetLogLevel(level);
-			}
-		}
-		return true;
-	}
+
+	// "<key>" なら "<key>*" 全部に展開する。
+#define EXPAND(key)	do {	\
+	if (name == key) {	\
+		for (auto obj : gMainApp.GetObjects()) {	\
+			if (strncasecmp(obj->GetName().c_str(), key, strlen(key)) == 0)	\
+				obj->SetLogLevel(level);	\
+		}	\
+		return true;	\
+	}	\
+} while (0)
+	EXPAND("bankram");
+	EXPAND("fdd");
+	EXPAND("gfpic");
+	EXPAND("hostcom");
+	EXPAND("hostnet");
 
 	// エイリアスリストを作る
 	using aliaslist_t = std::vector<std::pair<std::string, Object *>>;
