@@ -29,15 +29,15 @@ static inline uint32
 acc_cmp(uint32 a, uint32 b)
 {
 	uint32 rv = 0;
-	if (a >= b) rv |= (1 << 11);
-	if (a <  b) rv |= (1 << 10);
-	if (a <= b) rv |= (1 << 9);
-	if (a >  b) rv |= (1 << 8);
-	if ((int32)a >= (int32)b) rv |= (1 << 7);
-	if ((int32)a <  (int32)b) rv |= (1 << 6);
-	if ((int32)a <= (int32)b) rv |= (1 << 5);
-	if ((int32)a >  (int32)b) rv |= (1 << 4);
-	if ((int32)a != (int32)b) rv |= (1 << 3);
-	if ((int32)a == (int32)b) rv |= (1 << 2);
+	if (a >= b) rv |= (1U << 11);
+	if (a <  b) rv |= (1U << 10);
+	if (a <= b) rv |= (1U << 9);
+	if (a >  b) rv |= (1U << 8);
+	if ((int32)a >= (int32)b) rv |= (1U << 7);
+	if ((int32)a <  (int32)b) rv |= (1U << 6);
+	if ((int32)a <= (int32)b) rv |= (1U << 5);
+	if ((int32)a >  (int32)b) rv |= (1U << 4);
+	if ((int32)a != (int32)b) rv |= (1U << 3);
+	if ((int32)a == (int32)b) rv |= (1U << 2);
 	return rv;
 }
