@@ -4,6 +4,27 @@
 // Licensed under nono-license.txt
 //
 
+// 例外処理について。
+// 例外のうち、リセット(優先度1)、命令アクセス(2)、未実装命令(3)、特権違反(4)
+// とその他の命令内部発生例外(5、トラップやゼロ除算など) は、この命令に
+// 直接起因しているため precise 例外に分類される。
+// 一方、割り込み(優先度6)、FPU imprecise(7)、データアクセス(8) は imprecise
+// 例外に分類される。
+//
+// データアクセス例外について。
+// Figure 6-3 参照。例外 (例えば割り込み例外) 起動時にデータユニットが稼働中
+// ならデータアクセスの完了を待つ。データアクセスが失敗(fault)すればその状態
+// でシャドウレジスタをフリーズするので、割り込みの受け付けとデータアクセス
+// 例外が同じ命令内で起きると、その命令直後の境界での例外はベクタは割り込み
+// 例外 (1番)だが DMTx が VALID という混合状態になる。
+// 実際 OpenBSD のカーネルはこの状態を認識してわざわざ処理している。
+//
+// 実機は、上記の内部例外を起こす整数ユニットとデータユニットが並行して
+// 動けるため、割り込み以外の例外でもデータアクセス例外が同時に起きるかも
+// 知れないが、ここでは考えない。
+// また割り込みと FPU imprecise も同時に起きる可能性はあるが未調査。
+// FPU imprecise とデータアクセスについても未調査。
+
 #include "m88100.h"
 #include "debugger.h"
 #include "mpu88xx0.h"
@@ -44,41 +65,37 @@ MPU88xx0Device::ExecNormal(Event& ev)
 
 	// この時点で実行しようとする命令が opX に入っている
 
-	if (OpIsBusErr(reg.opX)) {
+	if (__predict_false(OpIsBusErr(reg.opX))) {
 		// プリフェッチしていた命令がバスエラー
-		Exception(EXCEP_INST);
-		goto exit;
-	}
-
-	op12 = op32_to_12(reg.opX);
-	switch (op12) {
+		Exception(EXCPRI_INST);
+	} else {
+		op12 = op32_to_12(reg.opX);
+		switch (op12) {
 #include "m88100switch.inc"
-	 default:
-		OP_FUNC(illegal);
-		break;
+		 default:
+			OP_FUNC(illegal);
+			break;
+		}
 	}
 
-	if (__predict_false(intr_pending != 0)) {
-		if (resetting) {
+	// 例外発生か。
+	if (__predict_false(excep_pending != 0)) {
+		if (__predict_false(resetting)) {
 			// 命令実行の先でリセットが起きたら以降を全部キャンセルして帰る。
-			// システムコントローラのリセットポートで起きる。
-			// リセットは実際には intr_pending ではないが、
-			// ここでフラグ処理をまとめるために立ててある。
+			// システムコントローラのリセットポートアクセスで起きる。
 			return;
 		}
-		if (IsIntrEnable()) {
-			// 割り込み処理要求が来ていて、
-			// 直前の命令が割り込みを禁止していなければ割り込み。
-			// XXX 実際は違う気がする
-			putlog(4, "INTR take");
-			ExceptionCore(EXCEP_INTERRUPT, ExceptionKind::INTR);
-			goto exit;
-		}
+
+		// 内部(precise)例外のうち優先度の最も高いのを一つ実行。
+		// x & (-x) で立ってる最下位ビットだけにする。
+		uint32 pri = excep_pending & (-excep_pending);
+
+		excep_pending &= ~pri;
+		ExceptionCore(pri);
 	}
 
 	used_cycle++;
 
- exit:
 	Prefetch();
 
 	int cycle = used_cycle - cycle_start;
@@ -86,22 +103,68 @@ MPU88xx0Device::ExecNormal(Event& ev)
 	scheduler->StartEvent(ev);
 }
 
-// 通常の例外。
-// DataAccessException, エラー、割り込み、TRAP からは使わない。
-// vec はベクタ番号。
+// 例外処理。ExecNormal() の下請け。
+// 例外を発生させるのはこれではなく、この下にある *Exception() のほう。
 void
-MPU88xx0Device::Exception(int vec)
-{
-	ExceptionCore(vec, ExceptionKind::NORMAL);
-}
-
-// 例外処理コア部分
-// vec はベクタ番号。
-void
-MPU88xx0Device::ExceptionCore(int vec, ExceptionKind cause)
+MPU88xx0Device::ExceptionCore(uint32 pri)
 {
+	uint32 vec;
 	uint syscall = 0;
 
+	// 例外種別(優先度)からベクタ番号に変換。
+	uint primask = pri;
+	switch (pri) {
+	 case EXCPRI_ERROR:
+		vec = EXCVEC_ERROR;
+		break;
+	 case EXCPRI_INST:
+		vec = EXCVEC_INST;
+		break;
+	 case EXCPRI_UNIMPL_OP:
+		vec = EXCVEC_UNIMPL_OP;
+		break;
+	 case EXCPRI_PRIV:
+		vec = EXCVEC_PRIV;
+		break;
+	 case EXCPRI_INTERNAL:
+	 case EXCPRI_TRAP:
+		// ベクタは別に指定されている。
+		vec = excep_vector;
+		excep_vector = 0;
+		break;
+
+	 case EXCPRI_OUTER:
+		// 割り込みとデータアクセス例外は同時に処理する。
+		// OUTER は、割り込み、データアクセス、FPU imprecise の少なくとも
+		// どれかが立っているという代表値なので、ここで欲しいのは
+		// excep_pending の残っているビット (複数かも知れない) のほう。
+		// FPU imprecise は未調査。
+		primask = excep_pending;
+		excep_pending = 0;
+
+		pri = primask & (-primask);
+		switch (pri) {
+		 case EXCPRI_INTERRUPT:
+			vec = EXCVEC_INTERRUPT;
+			break;
+		 case EXCPRI_SFU_IMPRECISE:
+			vec = EXCVEC_SFU1_IMPRECISE;
+			break;
+		 case EXCPRI_DATA:
+			vec = EXCVEC_DATA;
+			break;
+		 default:
+			assert(false);
+			break;
+		}
+		break;
+
+	 default:
+	 case EXCPRI_RESET:	// 先に弾いてるので来ない。
+		assert(false);
+		break;
+	}
+
 	// 例外履歴に記録 (例外発生はブランチ履歴にも記録)。
 	// ベクタ 450 なら OpenBSD システムコール番号もついでに記録。
 	// システムコール番号は tb0 発行時点で r13 にセットされている。
@@ -109,7 +172,7 @@ MPU88xx0Device::ExceptionCore(int vec, ExceptionKind cause)
 	// 198番 __syscall(2) だと r2:r3 (なので実質 r3)。
 	// 非公式 NetBSD/luna88k はベクタ 128 を使っているようだ。
 	excep_counter[vec]++;
-	if (vec == 128 || vec == 450) {
+	if (vec == 450 || vec == 128) {
 		syscall = reg.r[13];
 		if (syscall == 0) {
 			syscall = reg.r[2];
@@ -125,20 +188,25 @@ MPU88xx0Device::ExceptionCore(int vec, ExceptionKind cause)
 	// デバッガに通知 (例外ブレーク用)
 	debugger->NotifyExceptionMain(vec);
 
-	// 命令中で起きる例外はとりあえず全部2クロック追加? (Table.7-5)
-	if (cause != ExceptionKind::INTR) {
-		AddCycle(2);
-	}
-
 	if ((reg.psr & PSR_SFRZ)) {
-		if (cause != ExceptionKind::TRAP) {
-			putlog(2, "SFRZ Error Exception");
-			vec = EXCEP_ERROR;
-			cause = ExceptionKind::ERROR;
+		if (pri != EXCPRI_TRAP) {
+			putlog(0, "Exception (primask=$%x) during SFRZ!", primask);
+
+			// エラー例外でもう一度報告。
+			// BranchHistory に繰り返しと判定されないよう to を変えておく。
+			pri = EXCPRI_ERROR;
+			vec = EXCVEC_ERROR;
+			info = 0xfc000000 | vec;
+			exhist.AddEntry(from, 1, info);
+			brhist.AddEntry(from, 1, info);
+			debugger->NotifyExceptionMain(vec);
+		} else {
+			putlog(0, "Trap during Exception");
 		}
 	} else {
-		putlog(2, "Exception %x", vec);
-		if (vec != 1) {
+		if (vec == EXCVEC_INTERRUPT) {
+			putlog(2, "Exception (Interrupt)");
+		} else {
 			putlog(1, "Exception %x x=%x n=%x f=%x",
 				vec, reg.xip, reg.nip, reg.fip);
 		}
@@ -146,64 +214,32 @@ MPU88xx0Device::ExceptionCore(int vec, ExceptionKind cause)
 		reg.snip = reg.nip;
 		reg.sfip = reg.fip;
 
-		reg.sxip |= SIP_V;
+		if (pri != EXCPRI_UNIMPL_OP) {
+			reg.sxip |= SIP_V;
+		}
 		reg.snip |= SIP_V;
 		reg.sfip |= SIP_V;
 
 		if (OpIsBusErr(reg.opX)) {
-			reg.sxip |= SIP_E;	// E bit
+			reg.sxip |= SIP_E;
 		}
 		if (OpIsBusErr(reg.opF)) {
-			reg.snip |= SIP_E;	// E bit
-		}
-
-		if (vec == EXCEP_UNIMPL_OP) {
-			reg.sxip &= ~SIP_V;
+			reg.snip |= SIP_E;
 		}
 	}
 
-	if (cause != ExceptionKind::DATA) {
-		// データアクセス例外以外は、
-		// データパイプラインはフラッシュ状態とする。
-		// ほんとうは違う。かも。
-		reg.dmt0 = 0;
+	if ((primask & EXCPRI_DATA)) {
+		// パイプラインが未実装なので、PROM の DAE ハンドラが無限ループに
+		// なるのをごまかすハックは vm/prom.cpp 参照。
 	} else {
-		// xip: ld/st/xmem
-		// nip: next
-		// fip: fetch
-
-		// PROM 1.20 は DAE が起きると、
-
-		// ROM挙動               CPU の RTE 挙動
-		// sfip = sxip
-		// sxip -> ~V (INVALID)  CPU はそもそも sxip を評価しない
-		// snip = 0 (INVALID)    CPU はこれを nop 相当に処理する
-		// rte
-
-		// してくる。そのまま読むと命令を再実行しようとしているように
-		// 読めるが、再実行すると無限ループになってしまう。
-		// DAE の時点で xip が進んでいることを期待して利用している?
-
-		// openbsd は
-		// xmem の write バスエラーを検知すると命令を再実行するために
-		// fip = nip, nip = xip して rte する。ということは xip = xmem に
-		// なっていることを期待している。
-
-		// というわけで
-		// ld で DataAccessException のときは、実CPUはすでに次の命令を
-		// 実行している?
-		// もう少し考えると、スコアボードでレジスタの干渉がなければ、
-		// 実行している?
-
-		// 仕方がないのでハックでごまかす。
-		// rte のほうも参照のこと。
-
-		if (IsSuper() && reg.xip >= 0x41000000 && reg.xip < 0x42000000) {
-			// 遅延例外機構がないので、ROM 合わせのハック。
-			reg.sxip = reg.snip;
-		}
+		// データアクセス例外でなければ DMT の V ビットはクリア。(Table.6-3)
+		reg.dmt0 &= ~DM_VALID;
+		reg.dmt1 &= ~DM_VALID;
+		reg.dmt2 &= ~DM_VALID;
 	}
 
+	// XXX Figure 6-3 のフローだとトラップ例外の時は EPSR を保存しない
+	// ようだが本当だろうか。本文はトラップ例外の記載が少ない。
 	reg.epsr = reg.psr;
 	SetPSR(reg.psr | PSR_SUPER | PSR_SFD1 | PSR_IND | PSR_SFRZ);
 
@@ -214,18 +250,56 @@ MPU88xx0Device::ExceptionCore(int vec, ExceptionKind cause)
 	// ちょっと冗長という気もする。
 
 	if (OpIsBusErr(reg.opF)) {
-		if (cause == ExceptionKind::ERROR) {
+		if (pri == EXCPRI_ERROR) {
 			// double bus fault
 			// 実機は無限ループ状態
 			PANIC("Infinite ERROR (double bus fault)");
 		} else {
-			ExceptionCore(EXCEP_ERROR, ExceptionKind::ERROR);
+			putlog(0, "Bus error on fetching $%08x (vector $%x)!",
+				reg.vbr + (vec << 3), vec);
+			Exception(EXCPRI_ERROR);
 		}
 	}
 }
 
-inline uint32
-MPU88xx0Device::Calcdmt(uint32 flag)
+// EXCPRI_* の内部例外発生を予約する。
+// 例外はこの場で処理するのではなく、命令境界で優先度の高い方から処理される
+// ので、発生時に出来るのは予約することだけ。
+void
+MPU88xx0Device::Exception(uint32 pri)
+{
+	excep_pending |= pri;
+}
+
+// 外部優先度の例外の発生を予約する。
+void
+MPU88xx0Device::OuterException(uint32 pri)
+{
+	excep_pending |= pri;
+	excep_pending |= EXCPRI_OUTER;
+}
+
+// 内部例外の発生を予約する。
+// vec は発生させる例外のベクタ番号。
+void
+MPU88xx0Device::InternalException(uint32 vec)
+{
+	excep_vector = vec;
+	Exception(EXCPRI_INTERNAL);
+}
+
+// トラップ例外の発生を予約する。
+// vec は発生させる例外のベクタ番号。
+void
+MPU88xx0Device::TrapException(uint32 vec)
+{
+	excep_vector = vec;
+	Exception(EXCPRI_TRAP);
+}
+
+// flag から DMT の共通部分を作って返す。
+uint32
+MPU88xx0Device::MakeDMT(uint32 flag)
 {
 	uint32 dmt;
 
@@ -238,152 +312,177 @@ MPU88xx0Device::Calcdmt(uint32 flag)
 	return dmt;
 }
 
-// データアクセス例外(Read)
-// ld.b, ld.h, ld.hu, ld 用
+// データアクセス例外(Read)、ld.[b,h,(w)] 用。
 void
 MPU88xx0Device::ReadDataException32(uint32 addr, uint32 flag)
 {
 	// ミスアラインドはこの例外より前にチェックされていて、
-	// ここにはアラインドしかこない。
-	reg.dma0 = addr & ~0x3;
-	reg.dmt0  = Calcdmt(flag);
+	// ここには (アクセスサイズでの) アラインドしかこない。
+
+	reg.dmt0  = MakeDMT(flag);
 	reg.dmt0 |= FLD_D << 7;
 	reg.dmt0 |= flag & DM_SIGNED;
 	reg.dmt0 |= (flag & DM_EN_MASK) >> (addr & 0x03);
+	reg.dma0 = addr & ~0x3;
 
 	reg.dmt1 &= ~DM_VALID;
 	reg.dmt2 &= ~DM_VALID;
+
 	putlog(1, "ReadDAE xip=%x addr=%x", reg.xip, addr);
-	ExceptionCore(EXCEP_DATA, ExceptionKind::DATA);
+	OuterException(EXCPRI_DATA);
 }
 
-// データアクセス例外(Read)
-// ld.d 用
+// データアクセス例外(Read)、ld.d 用。
+// どちらのワードで例外が起きても first_addr は第1ワードのアドレス。
 void
-MPU88xx0Device::ReadDataException64(uint32 addr, uint32 flag)
+MPU88xx0Device::ReadDataException64(uint32 first_addr, uint32 flag)
 {
 	// ミスアラインドはこの例外より前にチェックされていて、
 	// ここにはアラインドしかこない。
-	reg.dma0 = addr;
-	reg.dmt0  = Calcdmt(flag);
-	reg.dmt0 |= DM_DOUB1;		// ダブルワードの第1アクセスのときセット
-	reg.dmt0 |= FLD_D << 7;
-	reg.dmt0 |= DM_EN_MASK;		// 32ビット有効
 
-	reg.dma1 = reg.dma0 + 4;
-	reg.dmt1  = Calcdmt(flag);
-	reg.dmt1 |= FLD_D2 << 7;
-	reg.dmt1 |= DM_EN_MASK;		// 32ビット有効
+	uint32 dmt_2nd;
+	uint32 dma_2nd;
 
+	reg.dmt1 &= ~DM_VALID;
 	reg.dmt2 &= ~DM_VALID;
 
-	if ((flag & DM_DOUB1) == 0) {
-		// .d の2ワード目がバスエラー。第1ワードのアクセスは成功した。
-		// DMx0 が例外を起こしたアクセスを示す。
-		// Manual 6.7.3
-		reg.dma0 = reg.dma1;
-		reg.dmt0 = reg.dmt1;
-		reg.dmt1 &= ~DM_VALID;
+	// 先に2ワード目の情報を作っておく。
+	dmt_2nd  = MakeDMT(flag);
+	dmt_2nd |= FLD_D2 << 7;
+	dmt_2nd |= DM_EN_MASK;		// 32ビット有効
+	dma_2nd = first_addr + 4;
+
+	uint32 doub1 = (flag & DM_DOUB1);
+	if (doub1) {
+		// 1ワード目がバスエラーなら、DMx0 と DMx1 を使う。
+		reg.dmt0  = MakeDMT(flag);
+		reg.dmt0 |= doub1;
+		reg.dmt0 |= FLD_D << 7;
+		reg.dmt0 |= DM_EN_MASK;		// 32ビット有効
+		reg.dma0 = first_addr;
+
+		reg.dmt1 = dmt_2nd;
+		reg.dma1 = dma_2nd;
+	} else {
+		// 2ワード目がバスエラーなら、DMx0 に2ワード目の情報だけ。
+		reg.dmt0 = dmt_2nd;
+		reg.dma0 = dma_2nd;
 	}
 
-	putlog(1, "ReadDAE64 xip=%x addr=%x", reg.xip, addr);
-	ExceptionCore(EXCEP_DATA, ExceptionKind::DATA);
+	putlog(1, "ReadDAE64 xip=%x addr=%x", reg.xip, reg.dma0);
+	OuterException(EXCPRI_DATA);
 }
 
-// データアクセス例外(Write)
-// st.b, st.h, st.hu, st 用
+// データアクセス例外(Write)、st.[b,h,(w)] 用。
 void
 MPU88xx0Device::WriteDataException32(uint32 addr, uint32 flag)
 {
 	// ミスアラインドはこの例外より前にチェックされていて、
-	// ここにはアラインドしかこない。
-	reg.dma0 = addr & ~0x3;
-	reg.dmt0  = Calcdmt(flag);
-	reg.dmt0 |= FLD_D << 7;
+	// ここには (アクセスサイズでの) アラインドしかこない。
+
+	reg.dmt0  = MakeDMT(flag);
 	reg.dmt0 |= (flag & DM_EN_MASK) >> (addr & 0x03);
 	reg.dmt0 |= DM_WRITE;
+	reg.dma0 = addr & ~0x3;
 	reg.dmd0 = rD;
 
 	reg.dmt1 &= ~DM_VALID;
 	reg.dmt2 &= ~DM_VALID;
+
 	putlog(1, "WriteDAE xip=%x addr=%x", reg.xip, addr);
-	ExceptionCore(EXCEP_DATA, ExceptionKind::DATA);
+	OuterException(EXCPRI_DATA);
 }
 
-// データアクセス例外(Write)
-// st.d 用
+// データアクセス例外(Write)、st.d 用。
+// どちらのワードで例外が起きても first_addr は第1ワードのアドレス。
 void
-MPU88xx0Device::WriteDataException64(uint32 addr, uint32 flag)
+MPU88xx0Device::WriteDataException64(uint32 first_addr, uint32 flag)
 {
 	// ミスアラインドはこの例外より前にチェックされていて、
 	// ここにはアラインドしかこない。
-	reg.dma0 = addr & ~0x3;
-	reg.dmt0  = Calcdmt(flag);
-	reg.dmt0 |= DM_DOUB1;		// ダブルワードの第1アクセスのときセット
-	reg.dmt0 |= FLD_D << 7;
-	reg.dmt0 |= DM_EN_MASK;
-	reg.dmt0 |= DM_WRITE;
-	reg.dmd0 = rD;
 
-	reg.dma1 = reg.dma0 + 4;
-	reg.dmt1  = Calcdmt(flag);
-	reg.dmt1 |= FLD_D2 << 7;
-	reg.dmt1 |= DM_EN_MASK;
-	reg.dmt1 |= DM_WRITE;
-	reg.dmd1 = rD2;
+	uint32 dmt_2nd;
+	uint32 dma_2nd;
+	uint32 dmd_2nd;
 
+	reg.dmt1 &= ~DM_VALID;
 	reg.dmt2 &= ~DM_VALID;
 
-	if ((flag & DM_DOUB1) == 0) {
-		// .d の2ワード目がバスエラー。第1ワードのアクセスは成功した。
-		// DMx0 が例外を起こしたアクセスを示す。
-		// Manual 6.7.3
-		reg.dma0 = reg.dma1;
-		reg.dmt0 = reg.dmt1;
-		reg.dmd0 = reg.dmd1;
-		reg.dmt1 &= ~DM_VALID;
+	// 先に2ワード目の情報を作っておく。
+	dmt_2nd  = MakeDMT(flag);
+	dmt_2nd |= DM_EN_MASK;
+	dmt_2nd |= DM_WRITE;
+	dma_2nd = first_addr + 4;
+	dmd_2nd = rD2;
+
+	uint32 doub1 = (flag & DM_DOUB1);
+	if (doub1) {
+		// 1ワード目がバスエラーなら、DMx0 と DMx1 を使う。
+		reg.dmt0  = MakeDMT(flag);
+		reg.dmt0 |= DM_DOUB1;		// ダブルワードの第1アクセスのときセット
+		reg.dmt0 |= DM_EN_MASK;
+		reg.dmt0 |= DM_WRITE;
+		reg.dma0 = first_addr;
+		reg.dmd0 = rD;
+
+		reg.dmt1 = dmt_2nd;
+		reg.dma1 = dma_2nd;
+		reg.dmd1 = dmd_2nd;
+	} else {
+		// 2ワード目がバスエラーなら、DMx0 に2ワード目の情報だけ。
+		reg.dmt0 = dmt_2nd;
+		reg.dma0 = dma_2nd;
+		reg.dmd0 = dmd_2nd;
 	}
 
-	putlog(1, "WriteDAE64 xip=%x addr=%x", reg.xip, addr);
-	ExceptionCore(EXCEP_DATA, ExceptionKind::DATA);
+	putlog(1, "WriteDAE64 xip=%x addr=%x", reg.xip, reg.dma0);
+	OuterException(EXCPRI_DATA);
 }
 
 // データアクセス例外(xmem)
-// xmem には .d は無い
+// xmem は .bu と (.w) しかない。
 void
 MPU88xx0Device::XmemDataException(uint32 addr, uint32 flag)
 {
-	reg.dma0 = addr & ~0x3;
-	reg.dmt0  = Calcdmt(flag);
-	reg.dmt0 |= DM_LOCK;		// xmem でセット
-	reg.dmt0 |= FLD_D << 7;
-	reg.dmt0 |= flag & DM_SIGNED;
-	reg.dmt0 |= (flag & DM_EN_MASK) >> (addr & 0x03);
+	// ミスアラインドはこの例外より前にチェックされていて、
+	// ここには (アクセスサイズでの) アラインドしかこない。
 
-	reg.dma1 = reg.dma0;
-	reg.dmt1  = Calcdmt(flag);
-	reg.dmt1 |= DM_LOCK;		// xmem でセット
-	reg.dmt1 |= FLD_D << 7;
-	reg.dmt1 |= flag & DM_SIGNED;
-	reg.dmt1 |= (flag & DM_EN_MASK) >> (addr & 0x03);
-	reg.dmt1 |= DM_WRITE;
-	reg.dmd1 = rD;
+	uint32 dmt_rd;
+	uint32 dmt_wr;
+	uint32 dma;
+	uint32 dmd_wr;
 
+	reg.dmt1 &= ~DM_VALID;
 	reg.dmt2 &= ~DM_VALID;
 
-	// ミスアラインドはこの例外より前にチェックされていて、
-	// ここにはアラインドしかこない。
+	dmt_rd  = MakeDMT(flag);
+	dmt_rd |= DM_LOCK;
+	dmt_rd |= FLD_D << 7;
+	dmt_rd |= flag & DM_SIGNED;
+	dmt_rd |= (flag & DM_EN_MASK) >> (addr & 0x03);
+	dma = addr & ~0x3U;
+
+	dmt_wr = dmt_rd | DM_WRITE;
+	dmd_wr = rD;
+
 	if (cmmu[1]->fault_code != m88200::FAULT_CODE_BUSERR
 	 || cmmu[1]->acc_read) {
-		// アドレス変換フォルトまたは read 時点
-		// nop
+		// アドレス変換フォルトまたは read 時点のバスエラー。
+		reg.dmt0 = dmt_rd;
+		reg.dma0 = dma;
+
+		reg.dmt1 = dmt_wr;
+		reg.dma1 = dma;
+		reg.dmd1 = dmd_wr;
 	} else {
-		// xmem の write 時点
-		reg.dmt0 &= ~DM_VALID;
+		// xmem の write 時点のバスエラー。
+		reg.dmt0 = dmt_wr;
+		reg.dma0 = dma;
+		reg.dmd0 = dmd_wr;
 	}
 
 	putlog(1, "xmemDAE xip=%x addr=%x", reg.xip, addr);
-	ExceptionCore(EXCEP_DATA, ExceptionKind::DATA);
+	OuterException(EXCPRI_DATA);
 }
 
 // FPxS レジスタをセット
@@ -427,7 +526,10 @@ MPU88xx0Device::SetFPxS(uint32 *h, uint32 *l, double src, uint32 ts)
 		uint32 exp;
 		// src は普通の double なのでこの指数部を取り出す
 		uint32 double_biased = (u.h >> 20) & 0x7ff;
-		if (double_biased == 0x7ff) {
+		if (double_biased == 0) {
+			// Zero
+			exp = 0;
+		} else if (double_biased == 0x7ff) {
 			// Inf, NAN
 			exp = 0x7ff00000;
 		} else {
@@ -435,7 +537,7 @@ MPU88xx0Device::SetFPxS(uint32 *h, uint32 *l, double src, uint32 ts)
 			int float_biased = (double_biased - 1023) + 127;
 			// この 8bit 値を 11bit に符号拡張して所定の位置へ
 			exp = ((int32)(float_biased << 24)) >> 4;
-			exp &= 0x7ff00000;
+			exp &= ~0x80000000;
 		}
 
 		*h = (u.h & 0x800fffff) | exp;
@@ -551,7 +653,7 @@ MPU88xx0Device::FPPreciseException(uint32 cause)
 	reg.fppt = reg.opX & 0xffe0;		// Opcode, T1, T2, TD は opX と同じ位置
 	reg.fppt |= m88100opf_D(reg.opX);	// DEST
 
-	Exception(EXCEP_SFU1_PRECISE);
+	InternalException(EXCVEC_SFU1_PRECISE);
 }
 
 // FP Imprecise 例外。
@@ -575,7 +677,7 @@ MPU88xx0Device::FPImpreciseException(uint32 cause)
 	reg.fprh &= ~FPRH_RNDMODE;
 	reg.fprh |= (reg.fpcr & FPCR_RM) << 14;
 
-	Exception(EXCEP_SFU1_IMPRECISE);
+	OuterException(EXCPRI_SFU_IMPRECISE);
 }
 
 void
@@ -587,8 +689,9 @@ MPU88xx0Device::fpu_unimpl()
 
 OP_DEF(illegal)
 {
-	// 本来は不当命令例外?
+	// 不当命令例外。
 	putlog(0, "Illegal instruction %08x", (uint32)reg.opX);
+	Exception(EXCPRI_UNIMPL_OP);
 }
 
 //
