@@ -20,13 +20,13 @@
 #define O5		m88100opf_O5(reg.opX)
 #define IMM16	m88100opf_IMM16(reg.opX)
 #define VEC9	m88100opf_VEC9(reg.opX)
+#define IsCO	((reg.opX & 0x100))
+#define IsCI	((reg.opX & 0x200))
 
 // 符号拡張・シフト済みのディスプレースメント
 // XXX: (負数の右シフト)GCC only
 #define D16		(((int32)(int16)reg.opX) << 2)
 #define D26		(((int32)(reg.opX << 6) >> 4))
-#define IsCO	((reg.opX >> 8) & 1)
-#define IsCI	((reg.opX >> 9) & 1)
 
 // レジスタアクセス
 #define rD	(reg.r[FLD_D])
@@ -54,6 +54,7 @@ do {	\
 	}	\
 } while (0)
 
+// n==0 は事前に弾いてある。
 #define FP_SET(t, n, v)	\
 do {	\
 	if (t == 0) {	\
@@ -150,7 +151,7 @@ MPU88xx0Device::DoBranch(uint32 toaddr, bool next_exec)
 inline uint32
 MPU88xx0Device::ldst_scale(uint32 size)
 {
-	if ((reg.opX & (1 << 9)) != 0) {
+	if ((reg.opX & (1U << 9)) != 0) {
 		return rS1 + rS2 * size;
 	} else {
 		return rS1 + rS2;
@@ -163,7 +164,7 @@ MPU88xx0Device::ldst_scale(uint32 size)
 #define LDST_USR_SCALE(size) do {				\
 	usr = (reg.opX & (1U << 8)) ? DM_USR : 0;	\
 	if (IsUser() && usr) {						\
-		Exception(EXCEP_PRIV);					\
+		Exception(EXCPRI_PRIV);					\
 		return;									\
 	}											\
 	addr = ldst_scale(size);					\
@@ -177,7 +178,7 @@ MPU88xx0Device::ldst_scale(uint32 size)
 		if (IsMXM()) {							\
 			addr &= ~(size - 1);				\
 		} else {								\
-			Exception(EXCEP_MISALIGNED);		\
+			InternalException(EXCVEC_MISALIGNED);	\
 			return;								\
 		}										\
 	}											\
@@ -275,9 +276,9 @@ acc_bf(int w, int ofs)
 {
 	uint32 m;
 	if (w == 0) {
-		m = 0xFFFFFFFFU;
+		m = 0xffffffffU;
 	} else {
-		m = ~(0xFFFFFFFFU << w);
+		m = ~(0xffffffffU << w);
 	}
 	return m << ofs;
 }
@@ -308,7 +309,7 @@ OP_DEF(xmem_bu_imm)
 	lastaddr = addr;
 	tmp = cmmu[1]->xmem_1(addr, rD & 0xff);
 	if ((int64)tmp < 0) {
-		XmemDataException(addr, DM_BU | DM_LOCK);
+		XmemDataException(addr, DM_BU);
 		return;
 	}
 	if (FLD_D == 0)
@@ -330,7 +331,7 @@ OP_DEF(xmem_w_imm)
 	lastaddr = addr;
 	tmp = cmmu[1]->xmem_4(addr, rD);
 	if ((int64)tmp < 0) {
-		XmemDataException(addr, DM_W | DM_LOCK);
+		XmemDataException(addr, DM_W);
 		return;
 	}
 	if (FLD_D == 0)
@@ -386,7 +387,8 @@ OP_DEF(ld_bu_imm)
 OP_DEF(ld_d_imm)
 {
 	uint32 addr;
-	uint64 data1, data2;
+	uint32 dae_addr;	// ReadDataException64() に渡すのは常に先頭アドレス。
+	uint64 data;
 
 	addr = rS1 + IMM16;
 	LDST_ALIGN(8);
@@ -394,21 +396,27 @@ OP_DEF(ld_d_imm)
 	AddCycle(4);	// Table.7-3
 
 	lastaddr = addr;
-	data1 = cmmu[1]->load_4(addr);
-	if ((int64)data1 < 0) {
-		ReadDataException64(addr, DM_D1);
+	dae_addr = addr;
+
+	data = cmmu[1]->load_4(addr);
+	if ((int64)data < 0) {
+		ReadDataException64(dae_addr, DM_D1);
 		return;
 	}
+	if (__predict_true(FLD_D != 0)) {
+		rD = data;
+	}
+
 	addr += 4;
-	data2 = cmmu[1]->load_4(addr);
-	if ((int64)data2 < 0) {
-		ReadDataException64(addr, DM_D2);
+	data = cmmu[1]->load_4(addr);
+	if ((int64)data < 0) {
+		ReadDataException64(dae_addr, DM_D2);
 		return;
 	}
 
-	rD  = data1;
-	rD2 = data2;
-	reg.r[0] = 0;
+	if (__predict_true(FLD_D2 != 0)) {
+		rD2 = data;
+	}
 }
 
 // 000101_DDDDDSSSSS_nnnnnn_nnnnnnnnnn
@@ -481,6 +489,7 @@ OP_DEF(ld_b_imm)
 OP_DEF(st_d_imm)
 {
 	uint32 addr;
+	uint32 dae_addr;	// WriteDataException64() に渡すのは常に先頭アドレス。
 	uint64 rv;
 
 	addr = rS1 + IMM16;
@@ -489,15 +498,18 @@ OP_DEF(st_d_imm)
 	AddCycle(1 + 4);	// Table.7-3
 
 	lastaddr = addr;
+	dae_addr = addr;
+
 	rv = cmmu[1]->store_4(addr, rD);
 	if ((int64)rv < 0) {
-		WriteDataException64(addr, DM_D1);
+		WriteDataException64(dae_addr, DM_D1);
 		return;
 	}
+
 	addr += 4;
 	rv = cmmu[1]->store_4(addr, rD2);
 	if ((int64)rv < 0) {
-		WriteDataException64(addr, DM_D2);
+		WriteDataException64(dae_addr, DM_D2);
 		return;
 	}
 }
@@ -682,7 +694,7 @@ OP_DEF(divu_imm)
 	AddCycle(38);	// Table.7-6 (ただしざっくり)
 
 	if (IMM16 == 0) {
-		Exception(EXCEP_INT_DIV);
+		InternalException(EXCVEC_INT_DIV);
 		return;
 	}
 	if (FLD_D == 0)
@@ -711,7 +723,7 @@ OP_DEF(add_imm)
 	uint32 imm = IMM16;
 	uint32 res = rS1 + imm;
 	if (isovf_add(rS1, imm, res)) {
-		Exception(EXCEP_INT_OVF);
+		InternalException(EXCVEC_INT_OVF);
 		return;
 	}
 	if (FLD_D == 0)
@@ -725,7 +737,7 @@ OP_DEF(sub_imm)
 	uint32 imm = IMM16;
 	uint32 res = rS1 - imm;
 	if (isovf_sub(rS1, imm, res)) {
-		Exception(EXCEP_INT_OVF);
+		InternalException(EXCVEC_INT_OVF);
 		return;
 	}
 	if (FLD_D == 0)
@@ -744,7 +756,7 @@ OP_DEF(div_imm)
 	AddCycle(38);	// Table.7-6 (ただしざっくり)
 
 	if ((int32)rS1 < 0 || IMM16 == 0) {
-		Exception(EXCEP_INT_DIV);
+		InternalException(EXCVEC_INT_DIV);
 		return;
 	}
 	if (FLD_D == 0)
@@ -764,14 +776,14 @@ OP_DEF(cmp_imm)
 OP_DEF(ldcr)
 {
 	if (IsUser()) {
-		Exception(EXCEP_PRIV);
+		Exception(EXCPRI_PRIV);
 		return;
 	}
 	if (FLD_CR > countof(reg.cr)) {
 		// 制御レジスタ番号が範囲外の時は
 		// 88100 では undocumented、
 		// 88110 では Unimplemented Opcode 例外。
-		Exception(EXCEP_UNIMPL_OP);
+		Exception(EXCPRI_UNIMPL_OP);
 		return;
 	}
 	if (FLD_D == 0)
@@ -815,18 +827,23 @@ OP_DEF(fldcr)
 OP_DEF(stcr)
 {
 	if (IsUser()) {
-		Exception(EXCEP_PRIV);
+		Exception(EXCPRI_PRIV);
 		return;
 	}
-	if (FLD_CR > countof(reg.cr)) {
+
+	uint n = FLD_CR;
+	if (n > countof(reg.cr)) {
 		// 制御レジスタ番号が範囲外の時は
 		// 88100 では undocumented、
 		// 88110 では Unimplemented Opcode 例外。
-		Exception(EXCEP_UNIMPL_OP);
+		Exception(EXCPRI_UNIMPL_OP);
 		return;
 	}
-	reg.cr[FLD_CR] = rS1;
-	if (FLD_CR == 1) {
+	// PID, SXIP は Read Only。
+	if (__predict_true(n != 0 && n != 4)) {
+		reg.cr[n] = rS1;
+	}
+	if (n == 1) {
 		SetPSR();
 	}
 }
@@ -872,23 +889,28 @@ OP_DEF(xcr)
 	uint32 tmp;
 
 	if (IsUser()) {
-		Exception(EXCEP_PRIV);
+		Exception(EXCPRI_PRIV);
 		return;
 	}
-	if (FLD_CR > countof(reg.cr)) {
+
+	uint n = FLD_CR;
+	if (n > countof(reg.cr)) {
 		// 制御レジスタ番号が範囲外の時は
 		// 88100 では undocumented、
 		// 88110 では Unimplemented Opcode 例外。
-		Exception(EXCEP_UNIMPL_OP);
+		Exception(EXCPRI_UNIMPL_OP);
 		return;
 	}
 	tmp = rS1;
-	rD = reg.cr[FLD_CR];
-	reg.cr[FLD_CR] = tmp;
-	if (FLD_CR == 1) {
-		SetPSR();
+	rD = reg.cr[n];
+	// PID, SXIP は Read Only。
+	if (__predict_true(n != 0 && n != 4)) {
+		reg.cr[n] = tmp;
 	}
 	reg.r[0] = 0;
+	if (n == 1) {
+		SetPSR();
+	}
 }
 
 // 100000_DDDDDSSSSS_11001n_nnnnnsssss
@@ -1027,10 +1049,10 @@ OP_DEF(fcmp)
 
 	uint32 res = 0;
 	if (s2 >= 0) {
-		if (s1 <= 0 || s2 <= s1) res |= 0x0800;	// ob
+		if (s1 <= 0 || s1 >= s2) res |= 0x0800;	// ob
 		if (0 <  s1 && s1 <  s2) res |= 0x0400;	// in
 		if (0 <= s1 && s1 <= s2) res |= 0x0200;	// ib
-		if (s1 < 0  || s2 <  s1) res |= 0x0100;	// ou
+		if (s1 < 0  || s1 >  s2) res |= 0x0100;	// ou
 	}
 	if (s1 >= s2) res |= 0x0080;		// ge
 	if (s1 <  s2) res |= 0x0040;		// lt
@@ -1156,7 +1178,7 @@ OP_DEF(bsr_n)
 // 110100_nnnnnSSSSS_nnnnnn_nnnnnnnnnn
 OP_DEF(bb0)
 {
-	if ((rS1 & (1 << B5)) == 0) {
+	if ((rS1 & (1U << B5)) == 0) {
 		AddCycle(1);	// Table.7-5
 		DoBranch(reg.xip + D16, false);
 	}
@@ -1165,7 +1187,7 @@ OP_DEF(bb0)
 // 110101_nnnnnSSSSS_nnnnnn_nnnnnnnnnn
 OP_DEF(bb0_n)
 {
-	if ((rS1 & (1 << B5)) == 0) {
+	if ((rS1 & (1U << B5)) == 0) {
 		DoBranch(reg.xip + D16, true);
 	}
 }
@@ -1173,7 +1195,7 @@ OP_DEF(bb0_n)
 // 110110_nnnnnSSSSS_nnnnnn_nnnnnnnnnn
 OP_DEF(bb1)
 {
-	if ((rS1 & (1 << B5)) != 0) {
+	if ((rS1 & (1U << B5)) != 0) {
 		AddCycle(1);	// Table.7-5
 		DoBranch(reg.xip + D16, false);
 	}
@@ -1182,7 +1204,7 @@ OP_DEF(bb1)
 // 110111_nnnnnSSSSS_nnnnnn_nnnnnnnnnn
 OP_DEF(bb1_n)
 {
-	if ((rS1 & (1 << B5)) != 0) {
+	if ((rS1 & (1U << B5)) != 0) {
 		DoBranch(reg.xip + D16, true);
 	}
 }
@@ -1276,11 +1298,11 @@ OP_DEF(rot_1)
 OP_DEF(tb0)
 {
 	if (IsUser() && VEC9 < 128) {
-		Exception(EXCEP_PRIV);
+		Exception(EXCPRI_PRIV);
 		return;
 	}
-	if ((rS1 & (1 << B5)) == 0) {
-		ExceptionCore(VEC9, ExceptionKind::TRAP);
+	if ((rS1 & (1U << B5)) == 0) {
+		TrapException(VEC9);
 	}
 }
 
@@ -1288,11 +1310,11 @@ OP_DEF(tb0)
 OP_DEF(tb1)
 {
 	if (IsUser() && VEC9 < 128) {
-		Exception(EXCEP_PRIV);
+		Exception(EXCPRI_PRIV);
 		return;
 	}
-	if ((rS1 & (1 << B5)) != 0) {
-		ExceptionCore(VEC9, ExceptionKind::TRAP);
+	if ((rS1 & (1U << B5)) != 0) {
+		TrapException(VEC9);
 	} else if (pseudo_stop_enable) {
 
 		// OpenBSD の STOP エミュレーション。
@@ -1325,11 +1347,11 @@ OP_DEF(tb1)
 OP_DEF(tcnd)
 {
 	if (IsUser() && VEC9 < 128) {
-		Exception(EXCEP_PRIV);
+		Exception(EXCPRI_PRIV);
 		return;
 	}
 	if ((acc_cnd(rS1) & M5) != 0) {
-		ExceptionCore(VEC9, ExceptionKind::TRAP);
+		TrapException(VEC9);
 	}
 }
 
@@ -1352,7 +1374,7 @@ OP_DEF(xmem_bu)
 	LEAVE_USR();
 
 	if ((int64)tmp < 0) {
-		XmemDataException(addr, DM_BU | DM_LOCK | usr);
+		XmemDataException(addr, DM_BU | usr);
 		return;
 	}
 	if (FLD_D == 0)
@@ -1379,7 +1401,7 @@ OP_DEF(xmem_w)
 	LEAVE_USR();
 
 	if ((int64)tmp < 0) {
-		XmemDataException(addr, DM_W | DM_LOCK | usr);
+		XmemDataException(addr, DM_W | usr);
 		return;
 	}
 
@@ -1443,8 +1465,9 @@ OP_DEF(ld_bu)
 // 111101_DDDDDSSSSS_000100_xU000sssss
 OP_DEF(ld_d)
 {
-	uint64 data1, data2;
 	uint32 addr;
+	uint32 dae_addr;	// ReadDataException64() に渡すのは常に先頭アドレス。
+	uint64 data;
 	uint32 usr;
 
 	LDST_USR_SCALE_ALIGN(8);
@@ -1452,29 +1475,34 @@ OP_DEF(ld_d)
 	AddCycle(4);	// Table.7-3
 
 	lastaddr = addr;
+	dae_addr = addr;
 
 	// 例外安全のために都度 S/U ビットを制御
 	ENTER_USR();
-	data1 = cmmu[1]->load_4(addr);
+	data = cmmu[1]->load_4(addr);
 	LEAVE_USR();
 
-	if ((int64)data1 < 0) {
-		ReadDataException64(addr, DM_D1 | usr);
+	if ((int64)data < 0) {
+		ReadDataException64(dae_addr, DM_D1 | usr);
 		return;
 	}
+	if (__predict_true(FLD_D != 0)) {
+		rD = data;
+	}
+
 	addr += 4;
 
 	ENTER_USR();
-	data2 = cmmu[1]->load_4(addr);
+	data = cmmu[1]->load_4(addr);
 	LEAVE_USR();
 
-	if ((int64)data2 < 0) {
-		ReadDataException64(addr, DM_D2 | usr);
+	if ((int64)data < 0) {
+		ReadDataException64(dae_addr, DM_D2 | usr);
 		return;
 	}
-	rD  = data1;
-	rD2 = data2;
-	reg.r[0] = 0;
+	if (__predict_true(FLD_D2 != 0)) {
+		rD2 = data;
+	}
 }
 
 // 111101_DDDDDSSSSS_000101_xU000sssss
@@ -1560,6 +1588,7 @@ OP_DEF(st_d)
 {
 	uint64 rv;
 	uint32 addr;
+	uint32 dae_addr;	// WriteDataException64() に渡すのは常に先頭アドレス。
 	uint32 usr;
 
 	LDST_USR_SCALE_ALIGN(8);
@@ -1567,13 +1596,14 @@ OP_DEF(st_d)
 	AddCycle(1 + 4);	// Table.7-3
 
 	lastaddr = addr;
+	dae_addr = addr;
 
 	ENTER_USR();
 	rv = cmmu[1]->store_4(addr, rD);
 	LEAVE_USR();
 
 	if ((int64)rv < 0) {
-		WriteDataException64(addr, DM_D1 | usr);
+		WriteDataException64(dae_addr, DM_D1 | usr);
 		return;
 	}
 
@@ -1584,7 +1614,7 @@ OP_DEF(st_d)
 	LEAVE_USR();
 
 	if ((int64)rv < 0) {
-		WriteDataException64(addr, DM_D2 | usr);
+		WriteDataException64(dae_addr, DM_D2 | usr);
 		return;
 	}
 }
@@ -1806,7 +1836,7 @@ OP_DEF(divu)
 	AddCycle(38);	// Table.7-6 (ただしざっくり)
 
 	if (rS2 == 0) {
-		Exception(EXCEP_INT_DIV);
+		InternalException(EXCVEC_INT_DIV);
 		return;
 	}
 	if (FLD_D == 0)
@@ -1837,7 +1867,7 @@ OP_DEF(add)
 		res += GetCY();
 	}
 	if (isovf_add(rS1, rS2, res)) {
-		Exception(EXCEP_INT_OVF);
+		InternalException(EXCVEC_INT_OVF);
 		return;
 	}
 
@@ -1863,7 +1893,7 @@ OP_DEF(sub)
 		res += 1;
 	}
 	if (isovf_sub(rS1, rS2, res)) {
-		Exception(EXCEP_INT_OVF);
+		InternalException(EXCVEC_INT_OVF);
 		return;
 	}
 
@@ -1887,7 +1917,7 @@ OP_DEF(div)
 	AddCycle(38);	// Table.7-6 (ただしざっくり)
 
 	if ((int32)rS1 < 0 || (int32)rS2 <= 0) {
-		Exception(EXCEP_INT_DIV);
+		InternalException(EXCVEC_INT_DIV);
 		return;
 	}
 	if (FLD_D == 0)
@@ -2025,7 +2055,6 @@ OP_DEF(ff1)
 		}
 		rD = i;
 	}
-
 }
 
 // 111101_DDDDDzzzzz_111011_00000sssss
@@ -2056,19 +2085,19 @@ OP_DEF(rte)
 
 	SetPSR(reg.epsr);
 
-	// LUNA-88K の PROM 1.20 は sxip, snip, sfip を制御してくる。
-	// PROM 1.20 の DAE ハンドラは、
-	// sxip = INVALID
-	// snip = 0 (INVALID)
-	// sfip = original sxip
-	// で rte してくる。
-	// CPU は sxip は評価しない。snip は 0 なので、無効として nop 扱い。
-	// (実際はクロックが消費されるかも?)
-	// sfip が VALID なので、そこから実行を再開する。
+	// 本当は有効な SxIP から順次パイプラインを埋めるだけで、
+	// こんな条件分けになるわけではないのだが、分岐履歴とかのため。
+
+	if (__predict_false((reg.sfip & SIP_V) == 0)) {
+		// どうなる?
+		putlog(0, "rte: SFIP.V==0 (NOT IMPLEMENTED)");
+	}
+
 	if ((reg.snip & SIP_V)) {
 		DoBranch(reg.snip & SIP_MASK, false);
 		reg.fip = reg.sfip & SIP_MASK;
 	} else {
+		// PROM 1.20 の DAE ハンドラが snip の VALID を落として rte を呼ぶ。
 		DoBranch(reg.sfip & SIP_MASK, false);
 	}
 }
@@ -2077,7 +2106,7 @@ OP_DEF(rte)
 OP_DEF(tbnd_1)
 {
 	if (rS1 > rS2) {
-		Exception(EXCEP_BOUNDS);
+		InternalException(EXCVEC_BOUNDS);
 	}
 }
 
@@ -2085,7 +2114,7 @@ OP_DEF(tbnd_1)
 OP_DEF(tbnd_2)
 {
 	if (rS1 > IMM16) {
-		Exception(EXCEP_BOUNDS);
+		InternalException(EXCVEC_BOUNDS);
 	}
 }
 
