@@ -874,7 +874,7 @@ DMACDevice::MonitorReg4(TextScreen& screen,
 		if (names[i][0] == '-') {
 			screen.Puts(x + i * 4, y, TA::Disable, "---");
 		} else {
-			bool b = reg & (1 << (3 - i));
+			bool b = reg & (1U << (3 - i));
 			screen.Puts(x + i * 4, y, TA::OnOff(b), names[i]);
 		}
 	}
