@@ -584,7 +584,7 @@ FDCDevice::MonitorReg(TextScreen& screen,
 		if (names[i][0] == '-') {
 			screen.Puts(x + i * 4, y, TA::Disable, "---");
 		} else {
-			bool b = reg & (1 << (7 - i));
+			bool b = reg & (1U << (7 - i));
 			screen.Puts(x + i * 4, y, TA::OnOff(b), names[i]);
 		}
 	}
@@ -1357,7 +1357,7 @@ FDCDevice::XferStartCallback(Event& ev)
 			{
 				putlog(3, "Transfer decided");
 				xfer_start = true;
-				sect_remain = 1 << (7 + resbuf.data.n);
+				sect_remain = 1U << (7 + resbuf.data.n);
 				if (cmdbuf.data.n == 0) {
 					xfer_remain = cmdbuf.data.dtl;
 					if (xfer_remain > 128) {
