@@ -441,7 +441,7 @@ Keyboard::GetKeyName(uint keycode)
 Keyboard::keyname {
 	"(none)",		// 00
 	"BREAK",		// 01
-	"COPY",			// 02
+	NULL,			// 02
 	"romaji",		// 03
 	"code",			// 04
 	"kigou",		// 05
@@ -459,7 +459,7 @@ Keyboard::keyname {
 	"ESC",			// 10
 	"BS",			// 11
 	"enter",		// 12
-	"INS",			// 13
+	NULL,			// 13
 	"space",		// 14
 	"DEL",			// 15
 	"XFER",			// 16
@@ -473,8 +473,8 @@ Keyboard::keyname {
 	"right",		// 1e
 	"down",			// 1f
 
-	"ROLLUP",		// 20
-	"ROLLDOWN",		// 21
+	"INS",			// 20
+	"COPY",			// 21
 	"1",			// 22
 	"2",			// 23
 	"3",			// 24
@@ -490,8 +490,8 @@ Keyboard::keyname {
 	"backslash",	// 2e
 	"<buzzer>",		// 2f
 
-	"OPT1",			// 30
-	"OPT2",			// 31
+	"CUT",			// 30
+	"PASTE",		// 31
 	"Q",			// 32
 	"W",			// 33
 	"E",			// 34
@@ -507,7 +507,7 @@ Keyboard::keyname {
 	"hira",			// 3e
 	"zenkaku",		// 3f
 
-	"HOME",			// 40
+	NULL,			// 40
 	"UNDO",			// 41
 	"A",			// 42
 	"S",			// 43
@@ -521,8 +521,8 @@ Keyboard::keyname {
 	"semicolon",	// 4b
 	"colon",		// 4c
 	"bracketright",	// 4d
-	NULL,			// 4e
-	NULL,			// 4f
+	"OPT1",			// 4e
+	"OPT2",			// 4f
 
 	"XF1",			// 50
 	"XF2",			// 51
@@ -541,7 +541,7 @@ Keyboard::keyname {
 	"XF4",			// 5e
 	"XF5",			// 5f
 
-	NULL,			// 60
+	"HOME",			// 60
 	"PAD_plus",		// 61
 	"PAD_minus",	// 62
 	"PAD_7",		// 63
@@ -558,8 +558,8 @@ Keyboard::keyname {
 	"PAD_enter",	// 6e
 	"PAD_CLR",		// 6f
 
-	NULL,			// 70
-	NULL,			// 71
+	"ROLLUP",		// 70
+	"ROLLDOWN",		// 71
 	"F1",			// 72
 	"F2",			// 73
 	"F3",			// 74
