@@ -42,6 +42,8 @@ LanceDevice::Init()
 	interrupt = GetInterruptDevice();
 	subram = GetSubRAMDevice();
 
+	hostnet->SetPortName("Lance");
+
 	// ホストからの受信コールバックを登録
 	scheduler->ConnectMessage(MessageID::HOSTNET_RX(0), this,
 		ToMessageCallback(&LanceDevice::RxMessage));
@@ -366,7 +368,7 @@ LanceDevice::MonitorReg(TextScreen& screen,
 	int x, int y, uint32 data, const char * const *names)
 {
 	for (int i = 0; i < 8; i++) {
-		bool b = data & (1 << (7 - i));
+		bool b = data & (1U << (7 - i));
 		screen.Puts(x + i * 5, y, TA::OnOff(b), names[i]);
 	}
 }
@@ -600,12 +602,12 @@ LanceDevice::ChipInit()
 	uint32 rdrp = initblock[8] | (initblock[9] << 16);
 	rmd_base = rdrp & 0xfffffc;
 	int rlen = rdrp >> 29;
-	rmd_num  = (1 << rlen);
+	rmd_num  = (1U << rlen);
 
 	uint32 tdrp = initblock[10] | (initblock[11] << 16);
 	tmd_base = tdrp & 0xfffffc;
 	int tlen = tdrp >> 29;
-	tmd_num  = (1 << tlen);
+	tmd_num  = (1U << tlen);
 
 	padr = MacAddr(
 		initblock[1] & 0xff,
