@@ -110,7 +110,8 @@
 #include "x68kkbd.h"
 
 // InsideOut p.135
-static const busdata wait = busdata::Wait(24 * 40_nsec);
+/*static*/ const busdata
+MFPDevice::wait = busdata::Wait(24 * 40_nsec);
 
 // コンストラクタ
 MFPDevice::MFPDevice()
@@ -606,7 +607,7 @@ MFPDevice::MonitorUpdate(Monitor *, TextScreen& screen)
 	screen.Print(0, y++, "<Interrupt>     %-6s %-4s %-4s %-4s",
 		"IER", "IMR", "IPR", "ISR");
 	for (int i = 15; i >= 0; i--, y++) {
-		uint16 mask = (1 << i);
+		uint16 mask = (1U << i);
 		screen.Print(0, y, "%c%u:%-12s %-6s %-4s %-4s %-4s",
 			i >= 8 ? 'A' : 'B',
 			(i % 8),
@@ -644,7 +645,7 @@ MFPDevice::MonitorUpdate(Monitor *, TextScreen& screen)
 	y = 9 + 8;
 	screen.Puts(x, y++, "<GPIP>    GPIP AER DDR");
 	for (int i = 7; i >= 0; i--, y++) {
-		uint mask = (1 << i);
+		uint mask = (1U << i);
 		screen.Print(x, y, "b%u %-6s  %u   %u   %s", i, gpipname[i],
 			(mfp_.gpip & mask) ? 1 : 0,
 			(mfp_.aer & mask) ? 1 : 0,
@@ -997,7 +998,7 @@ MFPDevice::SetVDisp(bool vdisp)
 	// TAI への入力はイベントカウントモードの時のみ
 	if (timer.IsEvent()) {
 		// AER が示す方向と同じエッジ方向の時
-		bool aer = (mfp.aer & (1 << MFP::GPIP_VDISP));
+		bool aer = (mfp.aer & (1U << MFP::GPIP_VDISP));
 		if (aer == vdisp) {
 			// カウンタを減算
 			timer.tdr--;
@@ -1033,8 +1034,8 @@ MFPDevice::SetAlarmOut(bool alarm)
 void
 MFPDevice::SetGPIP(int num, bool val)
 {
-	bool aer = (mfp.aer & (1 << num));
-	bool old = (mfp.gpip & (1 << num));
+	bool aer = (mfp.aer & (1U << num));
+	bool old = (mfp.gpip & (1U << num));
 
 	// 立ち上がりか立ち下がりで割り込みを上げる (AER による)
 	// AER OLD VAL   Interrupt
@@ -1061,9 +1062,9 @@ MFPDevice::SetGPIP(int num, bool val)
 	}
 
 	if (val) {
-		mfp.gpip |= (1 << num);
+		mfp.gpip |= (1U << num);
 	} else {
-		mfp.gpip &= ~(1 << num);
+		mfp.gpip &= ~(1U << num);
 	}
 }
 
