@@ -75,22 +75,23 @@ MPSCCDevice::MPSCCDevice()
 MPSCCDevice::~MPSCCDevice()
 {
 	if ((bool)hostcom) {
-		hostcom->SetRxCallback(NULL);
+		hostcom->ResetRxCallback();
 	}
 }
 
 bool
 MPSCCDevice::Create()
 {
-	// 対応するホストドライバを作成
+	// 対応するホストドライバを作成。
+	// ポート名は継承先ごとに Init() で設定している。
 	try {
-		hostcom.reset(new HostCOMDevice(this, "HostCOM"));
+		hostcom.reset(new HostCOMDevice(this, 0, ""));
 	} catch (...) { }
 	if ((bool)hostcom == false) {
 		warnx("Failed to initialize HostCOMDevice at %s", __method__);
 		return false;
 	}
-	hostcom->SetRxCallback(ToDeviceCallback(&MPSCCDevice::HostRxCallback));
+	hostcom->SetRxCallback(ToDeviceCallback(&MPSCCDevice::HostRxCallback), 0);
 
 	return true;
 }
@@ -111,7 +112,7 @@ MPSCCDevice::Init()
 {
 	interrupt = GetInterruptDevice();
 
-	scheduler->ConnectMessage(MessageID::HOSTCOM_RX, this,
+	scheduler->ConnectMessage(MessageID::HOSTCOM0_RX, this,
 		ToMessageCallback(&MPSCCDevice::RxMessage));
 
 	for (int ch = 0; ch < mpscc.chan.size(); ch++) {
@@ -678,7 +679,7 @@ MPSCCDevice::TxCallback(Event& ev)
 // ホストスレッドからの受信通知
 // (HostCOM スレッドで呼ばれる)
 void
-MPSCCDevice::HostRxCallback()
+MPSCCDevice::HostRxCallback(uint32 dummy)
 {
 	// XXX 暫定
 	// HostCom からメッセージが大量に送られると、スケジューラの
@@ -692,7 +693,7 @@ MPSCCDevice::HostRxCallback()
 
 	if (ev.IsRunning() == false) {
 		// スレッドを超えるためにメッセージを投げる
-		scheduler->SendMessage(MessageID::HOSTCOM_RX);
+		scheduler->SendMessage(MessageID::HOSTCOM0_RX);
 	}
 }
 
@@ -1086,7 +1087,7 @@ MPSCCDevice::GetParamStr(const MPSCCChan& chan) const
 	// 75 * 2 ^ (e-1) を求める
 	double a = (chan.baudrate / 75) * 1.5;
 	std::frexp(a, &e);
-	int b = 75 * (1 << (e - 1));
+	int b = 75 * (1U << (e - 1));
 
 	if (b * 19 / 20 <= chan.baudrate && chan.baudrate <= b * 21 / 20) {
 		baud = b;
@@ -1178,7 +1179,7 @@ MPSCCDevice::MonitorReg(TextScreen& screen,
 		if (names[i][0] == '-') {
 			screen.Puts(x + i * 5, y, TA::Disable, "----");
 		} else {
-			bool b = reg & (1 << (7 - i));
+			bool b = reg & (1U << (7 - i));
 			screen.Puts(x + i * 5, y, TA::OnOff(b), names[i]);
 		}
 	}
