@@ -569,7 +569,7 @@ class MPSCCDevice : public IODevice
 	virtual void Tx(int ch, uint32 data) = 0;
 
 	// シリアルから1バイト受信のメッセージとイベントコールバック
-	void HostRxCallback();
+	void HostRxCallback(uint32);
 	void RxMessage(MessageID, uint32);
 	void RxCallback(Event&);
 
