@@ -12,6 +12,7 @@
 #include "config.h"
 #include "debugger.h"
 #include "hd647180.h"
+#include "hd647180asci.h"
 #include "pio.h"
 #include "scheduler.h"
 #include "ssg.h"
@@ -72,6 +73,20 @@ MPU64180Device::putlogn(const char *fmt, ...) const
 	WriteLog(buf);
 }
 
+bool
+MPU64180Device::Create()
+{
+	try {
+		asci.reset(new HD647180ASCIDevice());
+	} catch (...) { }
+	if ((bool)asci == false) {
+		warnx("Failed to initialize HD647180ASCIDevice at %s", __method__);
+		return false;
+	}
+
+	return true;
+}
+
 // 初期化
 bool
 MPU64180Device::Init()
@@ -571,7 +586,7 @@ MPU64180Device::MonitorUpdateIO(Monitor *, TextScreen& screen)
 		screen.Puts(16 + 5 * i, y,
 			TA::OnOff(val & (1U << (7 - i))), rcr_str[i]);
 	}
-	screen.Print(46, y, "RefCyc=%u", (1 << (val & 3)) * 10);
+	screen.Print(46, y, "RefCyc=%u", (1U << (val & 3)) * 10);
 	y++;
 
 	y++;
