@@ -787,7 +787,7 @@ MPU68030Device::MonitorUpdateATC(Monitor *, TextScreen& screen)
 		bool tt_once_hit;
 
 		{
-			std::unique_lock<std::mutex> lock(table->atchist_mtx);
+			std::lock_guard<std::mutex> lock(table->atchist_mtx);
 			for (int i = 0; i < table->atchist.Length(); i++) {
 				const auto& s = table->atchist.Peek(i);
 				total += s.total;
@@ -847,7 +847,7 @@ MPU68030Device::CalcStat()
 
 		// こっちはロックする
 		{
-			std::unique_lock<std::mutex> lock(table->atchist_mtx);
+			std::lock_guard<std::mutex> lock(table->atchist_mtx);
 
 			table->atchist.EnqueueForce(s);
 			if (__predict_false(s.tthit != 0)) {
@@ -925,7 +925,7 @@ MPU68030Device::MonitorUpdateCache1(TextScreen& screen, int y,
 	// 統計。
 	float hit = 0.0;
 	{
-		std::unique_lock<std::mutex> lock(cache->hist_mtx);
+		std::lock_guard<std::mutex> lock(cache->hist_mtx);
 		int len = cache->hist.Length();
 		if (len != 0) {
 			for (int i = 0; i < cache->hist.Length(); i++) {
