@@ -155,7 +155,7 @@ class MPU680x0Device : public MainMPUDevice
 	// バスエラーからの復帰用。
 	inline void save_reg_pd(int n) {
 		// 該当ビットのフラグを立てる
-		flag_pd |= (1 << n);
+		flag_pd |= (1U << n);
 
 		// 保存
 		save_pd[n] = reg.A[n];
@@ -165,7 +165,7 @@ class MPU680x0Device : public MainMPUDevice
 	// バスエラーからの復帰用。
 	inline void save_reg_pi(int n) {
 		// 該当ビットのフラグを立てる
-		flag_pi |= (1 << n);
+		flag_pi |= (1U << n);
 
 		// 保存
 		save_pi[n] = reg.A[n];
@@ -174,14 +174,14 @@ class MPU680x0Device : public MainMPUDevice
 	// -(An) 実行前にレジスタを(まだ保存してなければ)保存する。
 	// 1命令で同じレジスタを2回 -(An) する可能性がある命令用。
 	inline void save_reg_pd_if(int n) {
-		if ((flag_pd & (1 << n)) == 0) {
+		if ((flag_pd & (1U << n)) == 0) {
 			save_reg_pd(n);
 		}
 	}
 	// (An)+ 実行前にレジスタを(まだ保存してなければ)保存する。
 	// 1命令で同じレジスタを2回 (An)+ する可能性がある命令用。
 	inline void save_reg_pi_if(int n) {
-		if ((flag_pi & (1 << n)) == 0) {
+		if ((flag_pi & (1U << n)) == 0) {
 			save_reg_pi(n);
 		}
 	}
