@@ -12,7 +12,8 @@
 #include "event.h"
 
 // InsideOut p.135
-static const busdata wait = busdata::Wait(7 * 40_nsec);
+/*static*/ const busdata
+PrinterDevice::wait = busdata::Wait(7 * 40_nsec);
 
 // コンストラクタ
 PrinterDevice::PrinterDevice()
