@@ -30,4 +30,7 @@ class PrinterDevice : public IODevice
 	busdata WritePort(uint32 offset, uint32 data);
 	busdata PeekPort(uint32 offset);
 	bool PokePort(uint32 offset, uint32 data);
+
+ private:
+	static const busdata wait;
 };
