@@ -197,7 +197,7 @@ Luna1PROMEmuDevice::ROM_InitMD()
 
 	// プレーン数(1,4,8)をプレーンマスク(0x01, 0x0f, 0xff)に
 	int nplane = bt45x->GetPlaneCount();
-	int planemask = (1 << nplane) - 1;
+	int planemask = (1U << nplane) - 1;
 	// プレーンマスク。実際は RAM に書き込まれるがとりあえず
 	roms.SetOffset(0xbc);
 	roms.Write4(planemask);
