@@ -70,6 +70,8 @@ RTL8019ASDevice::Init()
 {
 	uint n = GetId() - OBJ_ETHERNET0;
 
+	hostnet->SetPortName(string_format("Nereid#%u", n));
+
 	// MAC アドレスを取得
 	MacAddr macaddr;
 	if (inherited::GetConfigMacAddr(n, &macaddr, false) == false) {
@@ -552,7 +554,7 @@ RTL8019ASDevice::MonitorReg(TextScreen& screen,
 		if (names[i][0] == '-') {
 			screen.Puts(x + i * 6, y, TA::Disable, names[i] + 1);
 		} else {
-			bool b = reg & (1 << (7 - i));
+			bool b = reg & (1U << (7 - i));
 			screen.Puts(x + i * 6, y, TA::OnOff(b), names[i]);
 		}
 	}
