@@ -214,11 +214,11 @@ SCSIBus::AssertSEL(uint id)
 	putlog(4, "%s(#%u)", __func__, id);
 
 	// すでに上がってたら何もしない
-	if ((sel & (1 << id))) {
+	if ((sel & (1U << id))) {
 		return;
 	}
 
-	sel |= (1 << id);
+	sel |= (1U << id);
 
 	if (GetPhase() == SCSI::Phase::BusFree) {
 		// バスフリーフェーズなら、アービトレーションなしでセレクション
@@ -301,11 +301,11 @@ SCSIBus::NegateSEL(uint id)
 	putlog(4, "%s(#%u)", __func__, id);
 
 	// すでに下りてたら何もしない
-	if ((sel & (1 << id)) == 0) {
+	if ((sel & (1U << id)) == 0) {
 		return;
 	}
 
-	sel &= ~(1 << id);
+	sel &= ~(1U << id);
 
 	if (GetPhase() == SCSI::Phase::BusFree) {
 		// バスフリーフェーズで SEL を下げる
@@ -344,11 +344,11 @@ SCSIBus::AssertBSY(uint id)
 	putlog(4, "%s(#%u)", __func__, id);
 
 	// すでに上がってたら何もしない
-	if ((bsy & (1 << id))) {
+	if ((bsy & (1U << id))) {
 		return;
 	}
 
-	bsy |= (1 << id);
+	bsy |= (1U << id);
 
 	if (GetPhase() == SCSI::Phase::BusFree) {
 		// バスフリーフェーズだったらアービトレーション開始
@@ -390,11 +390,11 @@ SCSIBus::NegateBSY(uint id)
 	putlog(4, "%s(#%u)", __func__, id);
 
 	// すでに下りてたら何もしない
-	if ((bsy & (1 << id)) == 0) {
+	if ((bsy & (1U << id)) == 0) {
 		return;
 	}
 
-	bsy &= ~(1 << id);
+	bsy &= ~(1U << id);
 
 	if (GetPhase() == SCSI::Phase::Transfer) {
 		// 転送フェーズから BSY を下げたらバスフリーに移行。
