@@ -174,6 +174,7 @@ SCSITarget::ResetHard(bool poweron)
 		cmdseq.clear();
 		cmdlen = 0;
 		cmd.reset();
+		msgseq.clear();
 		bytecount = 0;
 		lastbyte = false;
 	}
@@ -218,11 +219,18 @@ SCSITarget::OnSelected()
 void
 SCSITarget::OnStartTransfer()
 {
-	// 最初はコマンドフェーズ。(ATN はとりあえず放置)
-	putlog(4, "Start Command Phase");
+	// ATN が立っていればメッセージアウトフェーズ。
+	// そうでなければコマンドフェーズ。
 	cmdseq.clear();
 	cmdlen = 0;
-	SetXfer(SCSI::XferPhase::Command);
+	msgseq.clear();
+	if (GetATN()) {
+		putlog(4, "Start MsgOut Phase by ATN");
+		SetXfer(SCSI::XferPhase::MsgOut);
+	} else {
+		putlog(4, "Start Command Phase");
+		SetXfer(SCSI::XferPhase::Command);
+	}
 	AssertREQ();
 }
 
@@ -325,6 +333,16 @@ SCSITarget::OnTransfer()
 				AssertRST();
 			}
 		}
+	} else if (xfer == SCSI::XferPhase::MsgOut) {
+		data = GetData();
+		putlog(4, "%s: msgseq[%02u] = $%02x", __func__,
+			(uint)msgseq.size(), data);
+		msgseq.push_back(data);
+		bytecount--;
+		if (bytecount == 0) {
+			lastbyte = true;
+		}
+
 	} else if (GetIO() == false) {
 		// OUT 方向 (ターゲットが受信側)
 		// bytecount は減算方向。
@@ -368,7 +386,23 @@ SCSITarget::OnTransferAck()
 			next = cmd->DoneDataIn();
 			break;
 		 case SCSI::XferPhase::MsgOut:
-			next = cmd->DoneMsgOut();
+			if (cmd) {
+				next = cmd->DoneMsgOut();
+			} else {
+				// コマンド前のメッセージアウト。
+				if (msgseq.size() == 1 && (msgseq[0] & 0x80)) {
+					putlog(1, "%s: MsgOut $%02x Identify (NOT SUPPORTED)",
+						__func__, msgseq[0]);
+				} else {
+					std::string buf;
+					for (auto m : msgseq) {
+						buf += string_format(" %02x", m);
+					}
+					putlog(0, "%s: MsgOut%s (NOT SUPPORTED)",
+						__func__, buf.c_str());
+				}
+				next = SCSI::XferPhase::Command;
+			}
 			break;
 		 case SCSI::XferPhase::MsgIn:
 			next = cmd->DoneMsgIn();
@@ -382,7 +416,7 @@ SCSITarget::OnTransferAck()
 		putlog(3, "%s done", SCSI::GetXferPhaseName(xfer));
 
 		// 簡略化のため、フェーズ完了時だけ REQ を引き伸ばすことができる
-		if (cmd->optime != 0) {
+		if (cmd && cmd->optime != 0) {
 			bus->SetOneshotReqWait(cmd->optime);
 			cmd->optime = 0;
 		}
@@ -404,6 +438,8 @@ SCSITarget::OnTransferAck()
 			// Command フェーズで Data フェーズの準備をすることが多いので
 			// 悩ましい。
 			switch (xfer) {
+			 case SCSI::XferPhase::Command:
+				break;
 			 case SCSI::XferPhase::DataOut:
 				cmd->buf.clear();
 				bytecount = cmd->recvbytes;
@@ -605,6 +641,9 @@ SCSIDisk::Init()
 	// 行わないといけない。うーん。
 	write_ignore = gConfig->Find(wikey).AsInt();
 
+	// writeignore が決まったので状態を初期化。
+	Clear();
+
 	// 設定時点でファイルが指定されていればオープン
 	if (filename.empty() == false) {
 		std::string path = gMainApp.NormalizePath(filename);
@@ -674,17 +713,29 @@ SCSIDisk::LoadDisk(const std::string& pathname_)
 		goto done;
 	}
 
-	// 書き込みモードをここで確定
+	// 書き込みモードをここで確定。
+	//
+	// デバイス特性	イメージ	write_ignore	write_mode
+	// ------------	----------	------------ :	----------
+	// 書込み可		書込み可	no				Writeable
+	// 〃			〃			yes				WriteIgnore
+	// 〃			書込み不可	no				WriteProtect (A)
+	// 〃			〃			yes				WriteIgnore  (B)
+	// 書込み不可	*			*				ReadOnly
+	//
+	// (A) は「ライトプロテクトした MO を入れた」状態。
+	// (B) は「書き換えたくないので書き込み権を落としてあるイメージファイルを
+	// あたかも書き込めるように見せたい」。(WI マークが出る)
 	if (IsWriteableDevice()) {
 		// 書き込み可能デバイスの場合
-		if (w_ok) {
-			if (write_ignore) {
-				write_mode = RW::WriteIgnore;
-			} else {
-				write_mode = RW::Writeable;
-			}
+		if (write_ignore) {
+			write_mode = RW::WriteIgnore;
 		} else {
-			write_mode = RW::WriteProtect;
+			if (w_ok) {
+				write_mode = RW::Writeable;
+			} else {
+				write_mode = RW::WriteProtect;
+			}
 		}
 	} else {
 		// 読み込み専用デバイスの場合
@@ -814,7 +865,16 @@ SCSIDisk::Clear()
 	medium_loaded = false;
 	image.Close();
 	pathname.clear();
-	write_mode = RW::Writeable;
+
+	if (IsWriteableDevice()) {
+		if (write_ignore) {
+			write_mode = RW::WriteIgnore;
+		} else {
+			write_mode = RW::Writeable;
+		}
+	} else {
+		write_mode = RW::ReadOnly;
+	}
 }
 
 // メディア状態変更を UI に通知する
