@@ -189,7 +189,7 @@ SPCDevice::ReadPort(uint32 offset)
 
 	switch (offset) {
 	 case SPC::BDID:
-		data = 1 << myid;
+		data = 1U << myid;
 		putlog(3, "BDID -> $%02x", data.Data());
 		break;
 
@@ -423,7 +423,7 @@ SPCDevice::PeekPort(uint32 offset)
 {
 	switch (offset) {
 	 case SPC::BDID:
-		return 1 << myid;
+		return 1U << myid;
 
 	 case SPC::SCTL:
 		return spc.sctl;
@@ -522,7 +522,7 @@ SPCDevice::MonitorUpdate(Monitor *, TextScreen& screen)
 #define A(n)	(baseaddr + (n) * regoffset)
 
 	y = 0;
-	screen.Print(0, y++, "$%08x BDID: $%02x (ID=%u)", A(0), (1 << myid), myid);
+	screen.Print(0, y++, "$%08x BDID: $%02x (ID=%u)", A(0), (1U << myid), myid);
 	screen.Print(0, y++, "$%08x SCTL: $%02x", A(1), sctl);
 	screen.Print(0, y++, "$%08x SCMD: $%02x", A(2), scmd);
 	screen.Print(0, y++, "$%08x INTS: $%02x", A(4), ints);
@@ -663,7 +663,7 @@ SPCDevice::MonitorUpdate(Monitor *, TextScreen& screen)
 	// BSY, SEL の内訳。"(76543210)" の順。
 	y = 2;
 	for (int i = 0; i < 8; i++) {
-		if ((bsy & (1 << i))) {
+		if ((bsy & (1U << i))) {
 			screen.Print(x + 4 + 8 - i, y, "%u", i);
 		} else {
 			screen.Print(x + 4 + 8 - i, y, "_");
@@ -671,7 +671,7 @@ SPCDevice::MonitorUpdate(Monitor *, TextScreen& screen)
 	}
 	y++;
 	for (int i = 0; i < 8; i++) {
-		if ((sel & (1 << i))) {
+		if ((sel & (1U << i))) {
 			screen.Print(x + 4 + 8 - i, y, "%u", i);
 		} else {
 			screen.Print(x + 4 + 8 - i, y, "_");
@@ -747,7 +747,7 @@ SPCDevice::MonitorReg(TextScreen& screen,
 		if (names[i][0] == '-') {
 			screen.Puts(x + i * 5, y, TA::Disable, "----");
 		} else {
-			bool b = reg & (1 << (7 - i));
+			bool b = reg & (1U << (7 - i));
 			screen.Puts(x + i * 5, y, TA::OnOff(b), names[i]);
 		}
 	}
@@ -1198,12 +1198,12 @@ SPCDevice::SelectCommand()
 	if (data == 0) {
 		// ID が一切立ってない?
 		idstr = "TEMP==0; No IDs?";
-	} else if ((data & (1 << myid)) == 0) {
+	} else if ((data & (1U << myid)) == 0) {
 		// 自 ID が立ってない?
 		idstr = string_format("TEMP=$%02x; No BDID?", data);
 	} else {
 		// ここで少なくとも自 ID は立っている
-		data &= ~(1 << myid);
+		data &= ~(1U << myid);
 
 		if (data == 0) {
 			// ターゲット ID がない
@@ -1356,7 +1356,7 @@ SPCDevice::SelectionTimeout(Event& ev)
 	if (loglevel >= 3) {
 		int tgtid;
 		uint8 data = GetData();
-		data &= ~(1 << myid);
+		data &= ~(1U << myid);
 		tgtid = SCSIBus::DecodeID(data);
 		if (tgtid == -1) {
 			putlogn("Selection timeout");
