@@ -1154,7 +1154,7 @@ VirtIOThread::~VirtIOThread()
 void
 VirtIOThread::Terminate()
 {
-	std::unique_lock<std::mutex> lock(mtx);
+	std::lock_guard<std::mutex> lock(mtx);
 	request |= REQ_TERMINATE;
 	cv.notify_one();
 }
@@ -1163,7 +1163,7 @@ VirtIOThread::Terminate()
 void
 VirtIOThread::RequestQueueNotify(uint idx)
 {
-	std::unique_lock<std::mutex> lock(mtx);
+	std::lock_guard<std::mutex> lock(mtx);
 	request |= REQ_QUEUE(idx);
 	cv.notify_one();
 }
