@@ -36,7 +36,7 @@ class VirtIONetDevice : public VirtIODevice, public IHWAddrFilter
 	const char *GetFeatureName(uint feature) const override;
 
 	// 受信
-	void HostRxCallback();
+	void HostRxCallback(uint32);
 	void RxMessage(MessageID, uint32);
 	void RxEvent(Event&);
 	void Rx(VirtQueue *);
