@@ -38,12 +38,8 @@ class WXSubWindow : public wxFrame
 	// フォントサイズ変更
 	virtual void FontChanged();
 
-	// ウィンドウサイズ再計算 (継承先による)
-	virtual void DoSize();
-
- protected:
-	// 縦リサイズ可能なテキストパネルの場合のレイアウト
-	bool LayoutTextVResize(const WXTextScreen *screen, int fixed_height = 0);
+	// コントロールからウィンドウサイズを設定する。
+	void Fit() override;
 
  private:
 	void OnClose(wxCloseEvent& event);
